#version 150
#moj_import <fog.glsl>
#moj_import <noise.glsl>

uniform sampler2D Sampler0;

uniform vec4 ColorModulator;
uniform float FogStart;
uniform float FogEnd;
uniform vec4 FogColor;
uniform float GameTime;

in vec2 texCoord0;
in float vertexDistance;
in vec4 vertexColor;
in vec4 normal;

out vec4 fragColor;

float fbm(vec3 x) {
	float v = 0.0;
	float a = 0.5;
	vec3 shift = vec3(100);

	for (int i = 0; i < 4; ++i) {
		v += a * snoise(x);
		x = x * 2.0 + shift;
		a *= 0.5;
	}

	return v;
}

void main() {
	vec4 color = vertexColor * ColorModulator;

	vec3 clouduv = vec3(texCoord0.x, (GameTime * 0.1), texCoord0.y);
	float cloud = clamp(fbm(clouduv * 16.0), 0.0, 1.0);
	if (cloud < 0.01) { discard; }
	vec3 shadow = mix(color.rgb, FogColor.rgb, clamp(pow(cloud - 0.1, 0.3), 0.0, 1.0));

	color.rgb = shadow;
	color.a *= clamp(cloud * 16.0, 0.0, 1.0);
	color.a *= linear_fog_fade(vertexDistance, FogStart, FogEnd);
	fragColor = color;
}
