#version 150
in vec2 texCoord;

uniform sampler2D DiffuseSampler;
uniform sampler2D PaletteSampler;
uniform vec2 OutSize;

out vec4 fragColor;

vec4 palettize(vec4 col) {
	float x = 0;
	float nd = distance(col, texture(PaletteSampler, vec2(0, 0)));

	for (int i = 1; i < 32; i++) {
		float d = distance(col, texture(PaletteSampler, vec2(float(i) / 32.0, 0.0)));

		if (d < nd) {
			nd = d;
			x = float(i) / 32.0;
		}
	}

	return texture(PaletteSampler, vec2(x, 0));
}

void main() {
	vec2 uv = texCoord;

	// comment out these to disable resolution scaling
	uv.x = floor(uv.x * (OutSize.x * 0.5)) / (OutSize.x * 0.5);
	uv.y = floor(uv.y * (OutSize.y * 0.5)) / (OutSize.y * 0.5);

	// uncomment this and comment out everything below to disable dithering
//	fragColor = palettize(texture(DiffuseSampler, uv));

	vec4 col = vec4(0);

	// remove the `* 0.5`s for full-resolution dithering
	// some monitors don't like per-pixel dither and can freak out
	bool x = int(gl_FragCoord.x * 0.5) % 2 == 0;
	bool y = int(gl_FragCoord.y * 0.5) % 2 == 0;

	col = texture(DiffuseSampler, uv);

	if ((x && !y) || (!x && y)) {
		col = palettize(col);
	} else {
		col = palettize(col - 0.05);
	}

	fragColor = col;
}
